"""Code for validating files from different sources"""
import logging
from typing import Dict, FrozenSet, List, Optional

from cg_hermes.config.pipelines import AnalysisType, Pipeline
from cg_hermes.config.tags import COMMON_TAG_CATEGORIES
from cg_hermes.deliverables import Deliverables
from cg_hermes.models.tags import TagMap

LOG = logging.getLogger(__name__)


def get_deliverables_obj(
    deliverables: Dict[str, List[Dict[str, str]]],
    pipeline: Pipeline,
    analysis_type: Optional[AnalysisType] = None,
) -> Deliverables:
    if pipeline == Pipeline.BALSAMIC and not analysis_type:
        LOG.info("Please specify analysis type for balsamic")
        raise SyntaxError
    return Deliverables(deliverables=deliverables, pipeline=pipeline, analysis_type=analysis_type)


def validate_common_tags() -> bool:
    """Validate the common tags"""
    for category in COMMON_TAG_CATEGORIES:
        tag_map: Dict[str, Dict[str, str]] = COMMON_TAG_CATEGORIES[category]
        for tag_name, value in tag_map.items():
            try:
                assert isinstance(tag_map[tag_name], dict)
            except AssertionError as err:
                LOG.warning("Tag %s in %s is on the wrong format", tag_name, category.upper())
                raise err
            try:
                assert "description" in value
            except AssertionError as err:
                LOG.warning("Tag %s in %s does not have a description", tag_name, category.upper())
                raise err
    return True


def validate_tag_map(tag_map: Dict[FrozenSet[str], dict]) -> bool:
    """Validate if a tag map is on the correct format"""
    for pipeline_tags, value in tag_map.items():
        assert isinstance(pipeline_tags, frozenset)
        TagMap.validate(value)
    return True



ALIGNMENT_COMMON = {
    "bam": {
        "description": "Alignment file in BAM format",
    },
    "bam-index": {
        "description": "Index file for alignment file in BAM format",
    },
    "bam-mt": {
        "description": "Alignment file in BAM format holding the reaads from chrMT",
    },
    "cram": {
        "description": "Alignment file in CRAM format",
    },
    "cram-index": {
        "description": "Index file for alignment file in CRAM format",
    },
}

RAW_DATA = {
    "fastq": {"description": "Files with raw data in fastq format"},
    "fasta": {"description": "Files with raw data in fasta format"},
    "forward-strand": {"description": "Reads from forward strand"},
    "reverse-strand": {"description": "Reads from reverse strand"},
    "unpaired-reads": {"description": "Reads that could not be paired"},
}


VARIANT_COMMON = {
    "cnv": {"description": "Copy number variants"},
    "normal": {"description": "Associated with normal sample"},
    "germline": {"description": "Associated with germline variants"},
    "somatic": {"description": "Associated with somatic variants"},
    "rhocall-viz": {"description": "Runs of homozygosity index"},
    "smn-calling": {"description": "Copy number calls for the SMN gene"},
    "snv": {"description": "Single nucleotide variants and short indels"},
    "snv-bcf": {"description": "Gvcf including all SNV variants"},
    "snv-gbcf": {"description": "Gvcf including all SNV variants"},
    "sv-bcf": {"description": "Gvcf including all SV variants"},
    "sv-vcf": {"description": "Variant call formated file with structural variants"},
    "sv-vcf-index": {"description": "Following index"},
    "telomere-calling": {"description": "Variants from telomere calling"},
    "tmb": {"description": "Tumor mutational burden information"},
    "tumor": {"description": "Associated with tumor sample"},
    "upd": {"description": "Uniparental disomy variants"},
    "vcf": {"description": "Variant call formated file"},
    "vcf-index": {"description": "Following index"},
    "vcf-snv-filtered": {"description": "SNV variants filtered by BALSAMIC"},
    "vcf-snv-filtered-index": {"description": "Following index"},
    "vcf-snv-clinical": {"description": "SNV variants from clinical panels"},
    "vcf-snv-clinical-index": {"description": "Following index"},
    "vcf-snv-research": {"description": "SNV variants from whole genome"},
    "vcf-snv-research-index": {"description": "Following index"},
    "vcf-str": {"description": "Short tandem repeat variants"},
    "vcf-str-index": {"description": "Following index"},
    "vcf-sv-clinical": {"description": "SV variants from clinical panels"},
    "vcf-sv-clinical-index": {"description": "Following index"},
    "vcf-sv-research": {"description": "SV variants from whole genome"},
    "vcf-sv-research-index": {"description": "Following index"},
    "vcf2cytosure": {"description": "Conversion from vcf format to cytosure format"},
    "variants": {"description": "File pertaining variant information in some way"},
}

FAMILY_COMMON = {
    "pedigree": {"description": "Pedigree information"},
    "ped": {"description": "Pedigree information"},
}

REPORTING_COMMON = {
    "csv": {"description": "Comma separated values"},
    "deliverable": {"description": "Deliverables file"},
    "delivery-report": {"description": "Delivery report with result for upload to Scout"},
    "multiqc-html": {"description": "Multiqc report for the run in html format"},
    "multiqc-json": {"description": "Multiqc report for the run in json format"},
    "pdf": {"description": "Portable document format"},
    "qc-report": {"description": "Results and QC"},
    "sambamba-depth": {"description": "Coverage information from sambamba"},
    "summary": {"description": "Overview file without detailed information"},
    "tsv": {"description": "Tab separated values"},
    "vcf-report": {"description": "Results and QC from variant calling"},
}

VALIDATIONS_COMMON = {
    "ped-check": {"description": "Results from pedigree validation"},
    "qc-metrics": {"description": "QC metrics from analysis"},
    "sex-check": {"description": "Results from sex validation"},
}

ANALYSIS_COMMON = {
    "assembly": {"description": "Assembly"},
    "autozyg": {"description": "Autozygous region"},
    "bed": {"description": "Bed file"},
    "bed-index": {"description": "Following index"},
    "bigwig": {"description": "Bigwig formated file"},
    "clinical": {"description": "Clinical subset"},
    "config": {"description": "Config file"},
    "coverage": {"description": "Output with coverage information"},
    "diagram": {"description": "Data diagram"},
    "filtered": {"description": "Filtered data"},
    "fracsnp": {"description": "Fraction of reads with snp"},
    "fusion": {"description": "Fusion transcripts"},
    "genes": {"description": "Related to genes"},
    "junction": {"description": "Junction data"},
    "metrics": {"description": "Data metrics"},
    "first-pass": {"description": "Data metrics first pass"},
    "second-pass": {"description": "Data metrics second pass"},
    "reference-info": {"description": "Information about references used in analysis"},
    "regions": {"description": "Output for regions"},
    "research": {"description": "Research set"},
    "segments": {"description": "Output for segments"},
    "sites": {"description": "Output for sites"},
    "tcov": {"description": "Coverage output"},
    "tiddit-coverage": {"description": "Coverage output from tiddit"},
    "visualization": {"description": "Visualizes data"},
}

TOOLS = {
    "arriba": {"description": "Fusion caller"},
    "asereadcounter": {"description": "Count reads mapping to heterozygous sites"},
    "chanjo": {"description": "Tool to keep track of coverage over specific regions"},
    "chromograph": {"description": "Tool to create PNG images from BED and WIG files from mikaell"},
    "cnvkit": {"description": "Tool to call copy number variations"},
    "ascatngs": {"description": "Tool to identify somatically acquired copy-number alterations"},
    "delly": {"description": "Cancer structural variant prediction tool"},
    "cyrius": {"description": "Tool to call the problematic CYP2D6 gene"},
    "deseq2": {"description": "Differential expression analysis with DESeq2"},
    "deepvariant": {"description": "Variantcaller"},
    "dnascope": {"description": "Call snv indels"},
    "genotyper": {"description": "SNV indel caller from sention"},
    "gffcompare": {"description": "Compare gff files"},
    "haplotype-caller": {"description": "Call snv and indels"},
    "manta": {"description": "Tool to call structural variants"},
    "mitodel": {"description": "Tool to identify mitochondrial deletion signatures"},
    "peddy": {"description": "Tool to check pedigree and ancestral relations"},
    "picard": {"description": "Picard set of bioinformatic tools"},
    "salmon-quant": {"description": "Transcript quantification"},
    "sention": {"description": "Sention algorithm"},
    "star-fusion": {"description": "Fusion caller"},
    "stringtie": {"description": "Transcript assembler"},
    "tnscope": {"description": "Call snv indels"},
    "tnscope-umi": {"description": "Call snv indels for umi"},
    "upd": {"description": "Uniparent disomy caller from bjhall"},
    "vardict": {"description": "Cancer variant caller"},
    "wisecondor": {"description": "NIPT caller"},
}

MIP_SPECIFIC = {
    "exe-ver": {"description": "Executable versions"},
    "mip-analyse": {"description": "MIP information about analysis"},
    "mip-config": {"description": "MIP configs for analysis"},
    "mip-log": {"description": "Logs for MIP analysis"},
    "pedigree-yaml": {"description": "YAML file with pedigree information"},
    "sample-info": {"description": "MIP info file about samples"},
}

BALSAMIC_SPECIFIC = {
    "balsamic-config": {"description": "Balsamic configs for analysis"},
    "balsamic-dag": {"description": "Balsamic run schema"},
    "balsamic-report": {"description": "Report from analysis"},
}

MICROSALT_SPECIFIC = {
    "microsalt-config": {"description": "Config settings for microsalt analysis"},
    "microsalt-log": {"description": "SLURM log for microsalt analysis"},
    "typing-report": {"description": "Results from bacterial typing"},
}

MUTANT_SPECIFIC = {
    "mutant-config": {"description": "Config settings for mutant analysis"},
    "mutant-log": {"description": "SLURM log for mutant analysis"},
    "pangolin": {"description": "Pangolin specific output"},
    "typing-report": {"description": "Results from typing"},
    "typing-summary": {"description": "Summary of results from typing"},
    "fohm-delivery": {"description": "Relevant for FoHM"},
    "ks-delivery": {"description": "Relevant for Karolinska hospital/microbial routine"},
    "komplettering": {"description": "Filetype specific for national uploading of microbial data"},
    "consensus": {"description": "Consensus sequence"},
    "ks-results": {"description": "KS modification of pangolin typing results"},
    "ks-aux-results": {"description": "KS modification of pangolin called variants"},
    "pangolin-typing": {"description": "Pangolin typing results"},
    "artic-sum": {"description": "GMS-Artic summary file"},
    "artic-var": {"description": "GMS-Artic variant file"},
    "artic-type": {"description": "GMS-Artic typing file"},
    "artic-qc": {"description": "GMS-Artic QC file"},
    "artic-json": {"description": "GMS-Artic json file"},
    "vcf-covid": {"description": "VCF file containing covid data. Distinction req. by production"},
    "instrument-properties": {"description": "Sequencing metadata"},
    "consensus-sample": {"description": "Separate sample consensus fasta"},
    "pangolin-typing-fohm": {"description": "Pangolin typing report with only qc_pass samples included"},
}

AVAILABLE_USAGES = {
    "audit",
    "cg",
    "chanjo",
    "deliver",
    "genotype",
    "nipt",
    "scout",
    "storage",
    "vogue",
}

COMMON_TAG_CATEGORIES = {
    "alignment_tags": ALIGNMENT_COMMON,
    "analysis_tags": ANALYSIS_COMMON,
    "family_tags": FAMILY_COMMON,
    "raw_data": RAW_DATA,
    "reporting_tags": REPORTING_COMMON,
    "tool_tags": TOOLS,
    "validation_tags": VALIDATIONS_COMMON,
    "variant_tags": VARIANT_COMMON,
}

ALL_TAG_CATEGORIES = [
    ALIGNMENT_COMMON,
    ANALYSIS_COMMON,
    BALSAMIC_SPECIFIC,
    FAMILY_COMMON,
    MIP_SPECIFIC,
    MICROSALT_SPECIFIC,
    MUTANT_SPECIFIC,
    RAW_DATA,
    REPORTING_COMMON,
    TOOLS,
    VALIDATIONS_COMMON,
    VARIANT_COMMON,
]

ALL_TAGS = {tag for category in ALL_TAG_CATEGORIES for tag in category.keys()}


if __name__ == "__main__":
    print(ALL_TAGS)



from typing import List


class Error(Exception):
    """Base class for exceptions in this module."""

    pass


class MissingFileError(Error):
    """Exception when mandatory file is missing."""

    def __init__(self, files: List[str], message: str):
        self.files = files
        self.message = message

    def __str__(self):
        return self.message



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cg_hermes', 'cg_hermes.cli', 'cg_hermes.config', 'cg_hermes.models']

package_data = \
{'': ['*']}

install_requires = \
['bump2version>=1.0.1,<2.0.0',
 'cgmodels>=0.4.0,<0.5.0',
 'coloredlogs>=14.0,<15.0',
 'pydantic>=1.7.3,<2.0.0',
 'pyyaml>=5.3.1,<6.0.0',
 'tabulate>=0.8.7,<0.9.0',
 'typer>=0.3.2,<0.4.0',
 'typing-extensions>=3.7.4,<4.0.0']

entry_points = \
{'console_scripts': ['hermes = cg_hermes.__main__:main']}

setup_kwargs = {
    'name': 'cg-hermes',
    'version': '1.6.5',
    'description': 'Convert information between pipelines and CG',
    'long_description': None,
    'author': 'Måns Magnusson',
    'author_email': 'mans.magnusson@scilifelab.se',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



"""Tag map for MIP-dna deliverables file"""

MIP_DNA_TAGS = {
    frozenset(["chanjo_sexcheck"]): {
        "tags": ["chanjo", "sex-check"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
    frozenset(["chromograph_upd", "sites"]): {
        "tags": ["chromograph", "upd", "sites"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
    frozenset(["chromograph_upd", "regions"]): {
        "tags": ["chromograph", "upd", "regions"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
    frozenset(["chromograph_cov", "tcov"]): {
        "tags": ["chromograph", "tcov"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
    frozenset(["chromograph_cov", "tcov"]): {
        "tags": ["chromograph", "tcov"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
    frozenset(["chromograph_rhoviz", "autozyg"]): {
        "tags": ["chromograph", "autozyg"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
    frozenset(["chromograph_rhoviz", "fracsnp"]): {
        "tags": ["chromograph", "fracsnp"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
    frozenset(["endvariantannotationblock", "clinical"]): {
        "tags": ["vcf-snv-clinical"],
        "index_tags": ["vcf-snv-clinical-index"],
        "is_mandatory": True,
        "bundle_id": True,
        "used_by": ["scout"],
    },
    frozenset(["endvariantannotationblock", "research"]): {
        "tags": ["vcf-snv-research"],
        "index_tags": ["vcf-snv-research-index"],
        "is_mandatory": True,
        "used_by": ["scout"],
    },
    frozenset(["expansionhunter", "sv_str"]): {
        "tags": ["vcf-str"],
        "index_tags": ["vcf-str-index"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
    frozenset(["glnexus_merge"]): {
        "tags": ["deepvariant","snv", "vcf"],
        "index_tags": ["deepvariant", "snv", "vcf-index"],
        "is_mandatory": False,
        "used_by": ["storage"],
    },
    frozenset(["markduplicates"]): {
        "tags": ["cram"],
        "index_tags": ["cram-index"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
    frozenset(["gatk_combinevariantcallsets"]): {
        "tags": ["snv-gbcf", "snv-bcf"],
        "index_tags": ["gbcf-index"],
        "is_mandatory": True,
        "used_by": ["genotype"],
    },
    frozenset(["mip_analyse", "config"]): {
        "tags": ["mip-analyse", "config"],
        "is_mandatory": True,
        "used_by": ["cg", "audit"],
    },
    frozenset(["mip_analyse", "config_analysis"]): {
        "tags": ["mip-config"],
        "is_mandatory": True,
        "used_by": ["cg", "audit"],
    },
    frozenset(["mip_analyse", "log"]): {
        "tags": ["mip-log"],
        "is_mandatory": True,
        "used_by": ["audit"],
    },
    frozenset(["mip_analyse", "pedigree"]): {
        "tags": ["pedigree-yaml"],
        "is_mandatory": True,
        "used_by": ["audit"],
    },
    frozenset(["mip_analyse", "pedigree_fam"]): {
        "tags": ["pedigree"],
        "is_mandatory": True,
        "used_by": ["scout"],
    },
    frozenset(["mip_analyse", "references_info"]): {
        "tags": ["mip-analyse", "reference-info"],
        "is_mandatory": True,
        "used_by": ["audit"],
    },
    frozenset(["mip_analyse", "sample_info"]): {
        "tags": ["sample-info"],
        "is_mandatory": True,
        "used_by": ["cg", "audit"],
    },
    frozenset(["multiqc_ar", "html"]): {
        "tags": ["multiqc-html"],
        "is_mandatory": True,
        "used_by": ["scout"],
    },
    frozenset(["multiqc_ar", "json"]): {
        "tags": ["multiqc-json"],
        "is_mandatory": True,
        "used_by": ["vogue"],
    },
    frozenset(["mitodel"]): {
        "tags": ["mitodel"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
    frozenset(["peddy_ar", "ped_check"]): {
        "tags": ["peddy", "ped-check"],
        "is_mandatory": True,
        "used_by": ["audit", "scout"],
    },
    frozenset(["peddy_ar", "peddy"]): {
        "tags": ["peddy", "ped"],
        "is_mandatory": True,
        "used_by": ["audit", "scout"],
    },
    frozenset(["peddy_ar", "sex_check"]): {
        "tags": ["peddy", "sex-check"],
        "is_mandatory": True,
        "used_by": ["audit", "scout"],
    },
    frozenset(["qccollect_ar"]): {
        "tags": ["qc-metrics"],
        "is_mandatory": True,
        "used_by": ["audit"],
    },
    frozenset(["rhocall_viz"]): {
        "tags": ["rhocall-viz"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
    frozenset(["sambamba_depth", "coverage"]): {
        "tags": ["coverage", "sambamba-depth"],
        "is_mandatory": True,
        "used_by": ["chanjo"],
    },
    frozenset(["samtools_subsample_mt"]): {
        "tags": ["bam-mt"],
        "index_tags": ["bam-mt-index"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
    frozenset(["smncopynumbercaller"]): {
        "tags": ["smn-calling"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
    frozenset(["star_caller"]): {
        "tags": ["cyrius"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
    frozenset(["sv_combinevariantcallsets"]): {
        "tags": ["sv-bcf"],
        "index_tags": ["sv-bcf-index"],
        "is_mandatory": True,
        "used_by": ["audit"],
    },
    frozenset(["sv_reformat", "clinical"]): {
        "tags": ["vcf-sv-clinical"],
        "index_tags": ["vcf-sv-clinical-index"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
    frozenset(["sv_reformat", "research"]): {
        "tags": ["vcf-sv-research"],
        "index_tags": ["vcf-sv-research-index"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
    frozenset(["telomerecat_ar"]): {
        "tags": ["telomere-calling"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
    frozenset(["tiddit_coverage"]): {
        "tags": ["tiddit-coverage", "bigwig"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
    frozenset(["upd_ar", "regions"]): {
        "tags": ["upd", "regions"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
    frozenset(["upd_ar", "sites"]): {
        "tags": ["upd", "sites"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
    frozenset(["version_collect_ar"]): {
        "tags": ["exe-ver"],
        "is_mandatory": True,
        "used_by": ["audit"],
    },
    frozenset(["vcf2cytosure_ar"]): {
        "tags": ["vcf2cytosure"],
        "is_mandatory": False,
        "used_by": ["scout"],
    },
}
